; CS-280 USARTDemo.asm
; Purpose: Demonstrates setting up the USART and transmitting a character.

.NOLIST
.INCLUDE "m32def.inc"		; contains .EQU's for DDRB, PORTB, and a lot more
.LIST

.EQU start = 0x2a			; define a symbol to represent address 0x2a in program memory

.CSEG						; this means further directives are for the Code Segment
.ORG 0x0
	rjmp start				; initialize Reset Vector

.ORG start					; define the beginning of the program - where the Assembler places further program instructions

;################################################################################# 
; Initialize the Stack Pointer, PORTB, PORTD
.DEF temp=r30
	ldi	temp, LOW(RAMEND)	; get low byte of the address of the highest byte in Data Memory we can use
	out SPL, temp			; load it into the low byte of SP
	ldi temp,HIGH(RAMEND)	; high byte of address of highest Data Memory byte
	out	SPH, temp			; load it into the high byte of SP

; configure PORTB for digital output
	ldi		temp, 0xff  	; set all bits to 1
	out 	DDRB, temp		; tell DDRB to configure pins as output (0xff means all output)
	ldi		temp, 0x00
	out		PORTB, temp		; turnn all led's off

	rcall 	delay60			; 60ms delay is more than enough to let the LCD initialize
	rcall 	initUSART		; initialize the USART

	ldi		temp, 'x'		; R0 is used as the "argument" to the transmit subroutine
	mov		R0, temp
repeat:
	rcall   delay1000		; pause between repeats
	rcall	transmit		; transmit a single character
	rjmp	repeat
	
; end init
;################################################################################# 

;################################################################################# 
; subroutine initUSART
initUSART:
	push temp
	; set baud rate via UBRRH and UBRRL
 	ldi temp, 0x00 ; sets URSEL to indicate UBRRH; bits 11:8 of UBRR to 0 
 	out UBRRH, TEMP 
 	ldi temp, 103  ; corresponds to 9600 bits/second
 	out UBRRL, TEMP ; sets UBRR bits 7:0

	; enable transmit and receive via UCSRB
 	ldi temp, 0x08 ; 0000 1000; disables interrupts, disables RX, enables TX
 	out UCSRB, TEMP 
 
	; set parity, #data bits, #stop bits, asynchrnous mode of operation via USSRC
 	ldi temp, 0x86 ; 1000 0110; sets URSEL, no parity, 1 stop bit, 8 databits
	out	UCSRC, temp

 	pop	temp
	ret
; end subroutine initUSART
;################################################################################# 

;################################################################################# 
; subroutine transmit
; places a character to be written in UDS.
; Register R0 is used as the "argument" containing the character to be written
transmit:
	
	push temp	; save temp register
	
; read UCSRA and poll TXC bit until set, indicating "ready to transmit"
read_udre:
	in 	R1, UCSRA 	; read UCSRA
 	sbrs R1, UDRE	; continue reading until bit UDRE is set
 	rjmp read_udre 
 
	; at this point, we're ready to transmit by loading a character into UDR
 	out UDR, R0 ; Write the letter to the display. 

	pop	temp	; restore temp register

	ret
; end subroutine blink
;################################################################################# 

;################################################################################# 
; subroutine delay60
; This subroutine invokes a 60ms delay
; set R0, R1, and R2 which are used as arguments to the Delay subroutine
; These particular values result in a 60ms delay at 16Mhz

delay60:
	push	R0		;save registers
	push    R1	
	push	R2	
	push	temp	

	ldi		temp, 32
	mov		R0, temp
	ldi		temp, 99
	mov		R1, temp
	ldi		temp, 100
	mov		R2, temp

	rcall 	delay			; 60ms delay is more than enough to let the LCD initialize
	
	pop		temp	; restore registers
	pop		R2
	pop		R1
	pop		R0	
	ret
; end subroutine delay60
;################################################################################# 

;################################################################################# 
; subroutine delay1000
; This subroutine invokes a 1000ms delay
; set R0, R1, and R2 which are used as arguments to the Delay subroutine
; These particular values result in a 1000ms delay at 16Mhz

delay1000:
	push	R0		;save registers
	push    R1	
	push	R2	
	push	temp	

	ldi		temp, 200
	mov		R0, temp
	ldi		temp, 200
	mov		R1, temp
	ldi		temp, 132
	mov		R2, temp

	rcall 	delay	; delay sub uses R0, R1, and R2 as "arguments"

	pop		temp	;restore registers
	pop		R2
	pop		R1
	pop		R0	
	ret
; end subroutine delay1000
;################################################################################# 


.INCLUDE "Delay.asm"
